import re
from ._patterns import COMPILED_PATTERNS, COMPILED_PATTERN_MAP, HEADER_RE, HEADER_MAP, REPLY_DATE_SPLIT_REGEX, STRIP_SPACE_CHARS

"""
Internal methods. For max_wrap_lines, min_header_lines, min_quoted_lines
documentation see the corresponding constants in _patterns.py.
"""

def find_pattern_on_line(lines, n, max_wrap_lines):
    """
    Finds a forward/reply pattern within the given lines on text on the given
    line number and returns a tuple with the type ('reply' or 'forward') and
    line number of where the pattern ends. The returned line number may be
    different from the given line number in case the pattern wraps over
    multiple lines.

    Returns (None, None) if no pattern was found.
    """
    for typ, regexes in COMPILED_PATTERN_MAP.items():
        for regex in regexes:
            for m in range(max_wrap_lines):
                match_line = ''.join(lines[n:n+1+m])
                if match_line.startswith('>'):
                    match_line = match_line[1:].strip()
                if re.match(regex, match_line.strip()):
                    return n+m, typ
    return None, None

def find_quote_position(lines, max_wrap_lines, limit=None):
    """
    Returns the (ending) line number of a quoting pattern. If a limit is given
    and the limit is reached, the limit is returned.
    """

    for n in range(len(lines)):
        end, typ = find_pattern_on_line(lines, n, max_wrap_lines)
        if typ:
            return end
        if limit != None and n >= limit-1:
            return n

    return None

def join_wrapped_lines(lines):
    """
    Join one or multiple lines that wrapped. Returns the reconstructed line.
    Takes into account proper spacing between the lines (see
    STRIP_SPACE_CHARS).
    """
    if len(lines) == 1:
        return lines[0]

    joined = lines[0]
    for line in lines[1:]:
        if joined and joined[-1] in STRIP_SPACE_CHARS:
            joined += line
        else:
            joined += ' '
            joined += line

    return joined

def extract_headers(lines, max_wrap_lines):
    """
    Extracts email headers from the given lines. Returns a dict with the
    detected headers and the amount of lines that were processed.
    """
    hdrs = {}
    header_name = None

    # Track overlong headers that extend over multiple lines
    extend_lines = 0

    lines_processed = 0

    for n, line in enumerate(lines):
        if not line.strip():
            header_name = None
            continue

        match = HEADER_RE.match(line)
        if match:
            header_name, header_value = match.groups()
            header_name = header_name.strip().lower()
            extend_lines = 0

            if header_name in HEADER_MAP:
                hdrs[HEADER_MAP[header_name]] = header_value.strip()
            lines_processed = n+1
        else:
            extend_lines += 1
            if extend_lines < max_wrap_lines and header_name in HEADER_MAP:
                hdrs[HEADER_MAP[header_name]] = join_wrapped_lines(
                        [hdrs[HEADER_MAP[header_name]], line.strip()])
                lines_processed = n+1
            else:
                # no more headers found
                break

    return hdrs, lines_processed

def parse_reply(line):
    """
    Parses the given reply line ("On DATE, USER wrote:") and returns a
    dictionary with the "Date" and "From" keys, or None, if couldn't parse.
    """
    if line.startswith('>'):
        line = line[1:].strip()

    date = user = None

    for pattern in COMPILED_PATTERN_MAP['reply']:
        match = pattern.match(line)
        if match:
            groups = match.groups()
            if len(groups) == 2:
                # We're lucky and got both date and user split up.
                date, user = groups
            else:
                split_match = REPLY_DATE_SPLIT_REGEX.match(groups[0])
                if split_match:
                    split_groups = split_match.groups()
                    date = split_groups[0]
                    user = split_groups[-1]
                else:
                    # Try a simple comma split
                    split = groups[0].rsplit(',', 1)
                    if len(split) == 2:
                        date, user = split

    if date:
        date = date.strip()

    if user:
        user = user.strip()

    if date and user:
        return {
            'date': date.strip(),
            'from': user.strip(),
        }

def find_unwrap_start(lines, max_wrap_lines, min_header_lines, min_quoted_lines):
    """
    Finds the starting point of a wrapped email. Returns a tuple containing
    (start_line_number, end_line_number, type), where type can be one of the
    following:

     * 'forward': A matching forwarding pattern was found
     * 'reply': A matching reply pattern was found
     * 'headers': Headers were found (usually a forwarded email)
     * 'quote': A quote was found

    start_line_number corresponds to the line number where the forwarding/reply
    pattern starts, or where the headers/quote starts. end_line_number is only
    different from start_line_number if the forwarding/reply pattern spans over
    multiple lines (it does not extend to the end of the headers or of the
    quoted section).

    Returns (None, None, None) if nothing was found.
    """

    for n, line in enumerate(lines):
        if not line.strip():
            continue

        # Find a forward / reply start pattern

        end, typ = find_pattern_on_line(lines, n, max_wrap_lines)
        if typ:
            return n, end, typ

        # Find a quote
        if line.startswith('>'):
            # Check if there are at least min_quoted_lines lines that match
            matched_lines = 1

            if matched_lines >= min_quoted_lines:
                return n, n, 'quoted'

            for peek_line in lines[n+1:]:
                if not peek_line.strip():
                    continue
                if not peek_line.startswith('>'):
                    break
                else:
                    matched_lines += 1
                if matched_lines >= min_quoted_lines:
                    return n, n, 'quoted'

        # Find a header
        match = HEADER_RE.match(line)
        if match:
            if len(extract_headers(lines[n:], max_wrap_lines)[0]) >= min_header_lines:
                return n, n, 'headers'

    return None, None, None


def unindent_lines(lines):
    unquoted = []
    for n, line in enumerate(lines):
        if line.startswith('> '):
            unquoted.append(line[2:])
        elif line.startswith('>'):
            unquoted.append(line[1:])
        else:
            break

    return unquoted

def unwrap(lines, max_wrap_lines, min_header_lines, min_quoted_lines):
    """
    Returns a tuple of:
    - Type ('forward', 'reply', 'headers', 'quoted')
    - Range of the text at the top of the wrapped message (or None)
    - Headers dict (or None)
    - Range of the text of the wrapped message (or None)
    - Range of the text below the wrapped message (or None)
    - Whether the wrapped text needs to be unindented
    """

    headers = {}

    # Get line number and wrapping type.
    start, end, typ = find_unwrap_start(lines, max_wrap_lines, min_header_lines, min_quoted_lines)

    # We found a line indicating that it's a forward/reply.
    if typ in ('forward', 'reply'):
        main_type = typ

        if typ == 'reply':
            reply_headers = parse_reply(join_wrapped_lines(lines[start:end+1]))
            if reply_headers:
                headers.update(reply_headers)

        # Find where the headers or the quoted section starts.
        # We can set min_quoted_lines to 1 because we expect a quoted section.
        start2, end2, typ = find_unwrap_start(lines[end+1:], max_wrap_lines, min_header_lines, 1)

        if typ == 'quoted':
            # Quoted section starts. Unindent and check if there are headers.
            quoted_start = end+1+start2
            unquoted = unindent_lines(lines[quoted_start:])
            rest_start = quoted_start + len(unquoted)
            start3, end3, typ = find_unwrap_start(unquoted, max_wrap_lines, min_header_lines, min_quoted_lines)
            if typ == 'headers':
                hdrs, hdrs_length = extract_headers(unquoted[start3:], max_wrap_lines)
                if hdrs:
                    headers.update(hdrs)
                rest2_start = quoted_start+start3+hdrs_length
                return main_type, (0, start), headers, (rest2_start, rest_start), (rest_start, None), True
            else:
                return main_type, (0, start), headers, (quoted_start, rest_start), (rest_start, None), True

        elif typ == 'headers':
            hdrs, hdrs_length = extract_headers(lines[start+1:], max_wrap_lines)
            if hdrs:
                headers.update(hdrs)
            rest_start = start + 1 + hdrs_length
            return main_type, (0, start), headers, (rest_start, None), None, False
        else:
            # Didn't find quoted section or headers, assume that everything
            # below is the qouted text.
            return main_type, (0, start), headers, (start+(start2 or 0)+1, None), None, False

    # We just found headers, which usually indicates a forwarding.
    elif typ == 'headers':
        main_type = 'forward'
        hdrs, hdrs_length = extract_headers(lines[start:], max_wrap_lines)
        rest_start = start + hdrs_length
        return main_type, (0, start), hdrs, (rest_start, None), None, False

    # We found quoted text. Headers may be within the quoted text.
    elif typ == 'quoted':
        unquoted = unindent_lines(lines[start:])
        rest_start = start + len(unquoted)
        start2, end2, typ = find_unwrap_start(unquoted, max_wrap_lines, min_header_lines, min_quoted_lines)
        if typ == 'headers':
            main_type = 'forward'
            hdrs, hdrs_length = extract_headers(unquoted[start2:], max_wrap_lines)
            rest2_start = start + hdrs_length
            return main_type, (0, start), hdrs, (rest2_start, rest_start), (rest_start, None), True
        else:
            main_type = 'quote'
            return main_type, (None, start), None, (start, rest_start), (rest_start, None), True



# HTML utils

import lxml.html
import lxml.etree

from ._patterns import FORWARD_LINE, FORWARD_STYLES, MULTIPLE_WHITESPACE_RE

INLINE_TAGS = ['a', 'b', 'em', 'i', 'strong', 'span', 'font', 'q',
               'object', 'bdo', 'sub', 'sup', 'center']

BEGIN = 'begin'
END = 'end'

try:
    string_class = basestring # Python 2.7
except NameError:
    string_class = str # Python 3.x

def trim_tree_after(element, include_element=True):
    """
    Removes the document tree following the given element. If include_element
    is True, the given element is kept in the tree, otherwise it is removed.
    """
    el = element
    for parent_el in element.iterancestors():
        el.tail = None
        if el != element or include_element:
            el = el.getnext()
        while el is not None:
            remove_el = el
            el = el.getnext()
            parent_el.remove(remove_el)
        el = parent_el

def trim_tree_before(element, include_element=True, keep_head=True):
    """
    Removes the document tree preceding the given element. If include_element
    is True, the given element is kept in the tree, otherwise it is removed.
    """
    el = element
    for parent_el in element.iterancestors():
        parent_el.text = None
        if el != element or include_element:
            el = el.getprevious()
        else:
            parent_el.text = el.tail
        while el is not None:
            remove_el = el
            el = el.getprevious()
            tag = remove_el.tag
            is_head = isinstance(tag, string_class) and tag.lower() == 'head'
            if not keep_head or not is_head:
                parent_el.remove(remove_el)
        el = parent_el

def trim_slice(lines, slice_tuple):
    """
    Trim a slice tuple (begin, end) so it starts at the first non-empty line
    (obtained via indented_tree_line_generator / get_line_info) and ends at the
    last non-empty line within the slice. Returns the new slice.
    """
    def _empty(line):
        return not line or line.strip() == '>'

    if not slice_tuple:
        return None

    slice_start, slice_end = slice_tuple

    if slice_start is None:
        slice_start = 0
    if slice_end is None:
        slice_end = len(lines)

    # Trim from beginning
    while slice_start < slice_end and _empty(lines[slice_start]):
        slice_start += 1

    # Trim from end
    while slice_end > slice_start and _empty(lines[slice_end-1]):
        slice_end -= 1

    return (slice_start, slice_end)

def unindent_tree(element):
    """
    Removes the outermost indent. For example, the tree
    "<div>A<blockqote>B<div>C<blockquote>D</blockquote>E</div>F</blockquote>G</div>"
    is transformed to
    "<div>A<div>B<div>C<blockquote>D</blockquote>E</div>F</div>G</div>"
    """
    for el in element.iter():
        if is_indentation_element(el):
            el.attrib.clear()
            el.tag = 'div'
            return

def slice_tree(tree, start_refs, end_refs, slice_tuple, html_copy=None):
    """
    Slices the HTML tree with the given start_refs and end_refs (obtained via
    get_line_info) at the given slice_tuple, a tuple (start, end) containing
    the start and end of the slice (or None, to start from the start / end at
    the end of the tree). If html_copy is specified, a new tree is constructed
    from the given HTML (which must be the equal to the original tree's HTML*).
    The resulting tree is returned.

    *) The reason we have to specify the HTML is that we can't reliably
       construct a copy of the tree using copy.copy() (see bug
       https://bugs.launchpad.net/lxml/+bug/1562550).
    """

    start_ref = None
    end_ref = None

    if slice_tuple:
        slice_start, slice_end = slice_tuple

        if ((slice_start is not None and slice_start >= len(start_refs)) or
            (slice_end is not None and slice_end <= 0)):
            return get_html_tree('')

        if slice_start != None and slice_start <= 0:
            slice_start = None

        if slice_end != None and slice_end >= len(start_refs):
            slice_end = None
    else:
        slice_start, slice_end = None, None

    if slice_start is not None:
        start_ref = start_refs[slice_start]

    if slice_end is not None:
        if slice_end < len(end_refs):
            end_ref = end_refs[slice_end-1]

    if html_copy is not None:
        et = lxml.etree.ElementTree(tree)

        new_tree = get_html_tree(html_copy)

        if start_ref:
            selector = et.getelementpath(start_ref[0])
            start_ref = (new_tree.find(selector), start_ref[1])

        if end_ref:
            selector = et.getelementpath(end_ref[0])
            end_ref = (new_tree.find(selector), end_ref[1])

    else:
        new_tree = tree

    if start_ref:
        trim_tree_before(start_ref[0],
                        include_element=(start_ref[1] == BEGIN))
    if end_ref:
        trim_tree_after(end_ref[0],
                       include_element=(end_ref[1] == END))

    return new_tree

def get_html_tree(html):
    """
    Given the HTML string, returns a LXML tree object. The tree is wrapped in
    <div> elements if it doesn't have a top level tag or parsing would
    otherwise result in an error. The wrapping can be later removed with
    strip_wrapping().
    """

    parser = lxml.html.HTMLParser(encoding='utf-8')
    html = html.encode('utf8')

    try:
        tree = lxml.html.fromstring(html, parser=parser)
    except lxml.etree.Error:
        # E.g. empty document. Use dummy <div>
        tree = lxml.html.fromstring('<div></div>')

    # If the document doesn't start with a top level tag, wrap it with a <div>
    # that will be later stripped out for consistent behavior.
    if tree.tag not in lxml.html.defs.top_level_tags:
        html = b'<div>%s</div>' % html
        tree = lxml.html.fromstring(html, parser=parser)

    return tree

def strip_wrapping(html):
    """
    Removes the wrapping that might have resulted when using get_html_tree().
    """
    if html.startswith('<div>') and html.endswith('</div>'):
        html = html[5:-6]
    return html.strip()

def render_html_tree(tree):
    """
    Renders the given HTML tree, and strips any wrapping that was applied in
    get_html_tree().

    You should avoid further processing of the given tree after calling this
    method because we modify namespaced tags here.
    """

    # HACK for Outlook emails, where tags like <o:p> are rendered as <p>.
    for el in tree.iter():
        if el.nsmap:
            el.tag = '{}:{}'.format(list(el.nsmap.keys())[0], el.tag)

    html = lxml.html.tostring(tree, encoding='utf8').decode('utf8')
    return strip_wrapping(html)

def is_indentation_element(element):
    if isinstance(element.tag, string_class):
        return element.tag.lower() == 'blockquote'
    return False

def tree_token_generator(el, indentation_level=0):
    """
    Internal generator that yields tokens for the given HTML element as
    follows:

    - A tuple (LXML element, BEGIN, indentation_level)
    - Text right after the start of the tag, or None.
    - Recursively calls the token generator for all child objects
    - A tuple (LXML element, END, indentation_level)
    - Text right after the end of the tag, or None.
    """

    if not isinstance(el.tag, string_class):
        return

    tag_name = el.tag.lower()

    is_indentation = is_indentation_element(el)

    if is_indentation:
        indentation_level += 1

    yield (el, BEGIN, indentation_level)

    yield el.text

    for child in el.iterchildren():
        for token in tree_token_generator(child, indentation_level):
            yield token

    if is_indentation:
        indentation_level -= 1

    yield (el, END, indentation_level)

    yield el.tail

def tree_line_generator(el, max_lines=None):
    """
    Internal generator that iterates through an LXML tree and yields a tuple
    per line. In this context, lines are blocks of text separated by <br> tags
    or by block elements. The tuples contain the following elements:

    - A tuple with the element reference (element, position) for the start
      of the line. The tuple consists of:
        - The LXML HTML element which references the line
        - Whether the text starts at the beginning of the referenced element,
          or after the closing tag
    - A similar tuple indicating the ending of the line.
    - The email indentation level, if detected.
    - The plain (non-HTML) text of the line

    If max_lines is specified, the generator stops after yielding the given
    amount of lines.

    For example, the HTML tree "<div>foo <span>bar</span><br>baz</div>" yields:

    - ((<Element div>, 'begin'), (<Element br>, 'begin'), 0, 'foo bar')
    - ((<Element br>, 'end'), (<Element div>, 'end'), 0, 'baz').

    To illustrate the indentation level, the HTML tree
    '<div><blockquote>hi</blockquote>world</div>' yields:

    - ((<Element blockquote>, 'begin'), (<Element blockquote>, 'end'), 1, 'hi')
    - ((<Element blockquote>, 'end'), (<Element div>, 'end'), 0, 'world')
    """

    def _trim_spaces(text):
        return MULTIPLE_WHITESPACE_RE.sub(' ', text).strip()

    # Buffer for the current line.
    line = ''

    # The reference tuple (element, position) for the start of the line.
    start_ref = None

    # The indentation level at the start of the line.
    start_indentation_level = None

    for token in tree_token_generator(el):
        if token is None:
            continue

        elif isinstance(token, tuple):
            el, state, indentation_level = token

            tag_name = el.tag.lower()

            line_break = (tag_name == 'br' and state == BEGIN)
            is_block = (tag_name not in INLINE_TAGS)
            is_forward = (is_block and state == BEGIN and
                          el.attrib.get('style') in FORWARD_STYLES)

            if is_block or line_break:
                line = _trim_spaces(line)

                if line or line_break or is_forward:
                    end_ref = (el, state)
                    yield start_ref, end_ref, start_indentation_level, line
                    line = ''

                    if is_forward:
                        # Simulate forward
                        yield (end_ref, end_ref, start_indentation_level,
                               FORWARD_LINE)

                if not line:
                    start_ref = (el, state)
                    start_indentation_level = indentation_level

        elif isinstance(token, string_class):
            line += token

        else:
            raise RuntimeError('invalid token: {}'.format(token))

    line = _trim_spaces(line)
    if line:
        yield line

def indented_tree_line_generator(el, max_lines=None):
    """
    Like tree_line_generator, but yields tuples (start_ref, end_ref, line),
    where the line already takes the indentation into account by having "> "
    prepended. If a line already starts with ">", it is escaped ("\\>"). This
    makes it possible to reliably use methods that analyze plain text to detect
    quoting.
    """
    gen = tree_line_generator(el, max_lines)
    for start_ref, end_ref, indentation_level, line in gen:
        # Escape line
        if line.startswith('>'):
            line = '\\' + line
        yield start_ref, end_ref, '> '*indentation_level + line

def get_line_info(tree, max_lines=None):
    """
    Shortcut for indented_tree_line_generator() that returns an array of
    start references, an array of corresponding end references (see
    tree_line_generator() docs), and an array of corresponding lines.
    """
    line_gen = indented_tree_line_generator(tree, max_lines=max_lines)
    line_gen_result = list(zip(*line_gen))
    if line_gen_result:
        return line_gen_result
    else:
        return [], [], []



# -*- coding: utf-8 -*-
# quotequail
# a library that identifies quoted text in email messages

import re

from . import _internal

__all__ = ['quote', 'quote_html', 'unwrap', 'unwrap_html']


def quote(text, limit=1000):
    """
    Takes a plain text message as an argument, returns a list of tuples. The
    first argument of the tuple denotes whether the text should be expanded by
    default. The second argument is the unmodified corresponding text.

    Example: [(True, 'expanded text'), (False, '> Some quoted text')]

    Unless the limit param is set to None, the text will automatically be quoted
    starting at the line where the limit is reached.
    """

    lines = text.split('\n')

    found = _internal.find_quote_position(lines, _patterns.MAX_WRAP_LINES)

    if found != None:
        return [(True, '\n'.join(lines[:found+1])), (False, '\n'.join(lines[found+1:]))]

    return [(True, text)]

def quote_html(html, limit=1000):
    """
    Like quote(), but takes an HTML message as an argument. The limit param
    represents the maximum number of lines to traverse until quoting the rest
    of the markup. Lines are separated by block elements or <br>.
    """
    from . import _html

    tree = _html.get_html_tree(html)

    start_refs, end_refs, lines = _html.get_line_info(tree, limit+1)

    found = _internal.find_quote_position(lines, 1, limit)

    if found == None:
        # No quoting found and we're below limit. We're done.
        return [(True, _html.render_html_tree(tree))]
    else:
        start_tree = _html.slice_tree(tree, start_refs, end_refs,
                                      (0, found+1), html_copy=html)
        end_tree   = _html.slice_tree(tree, start_refs, end_refs,
                                      (found+1, None))

    return [
        (True, _html.render_html_tree(start_tree)),
        (False, _html.render_html_tree(end_tree)),
    ]

def unwrap(text):
    """
    If the passed text is the text body of a forwarded message, a reply, or
    contains quoted text, a dictionary with the following keys is returned:

    - type: "reply", "forward" or "quote"
    - text_top: Text at the top of the passed message (if found)
    - text_bottom: Text at the bottom of the passed message (if found)
    - from / to / subject / cc / bcc / reply-to: Corresponding header of the
      forwarded message, if it exists. (if found)
    - text: Unindented text of the wrapped message (if found)

    Otherwise, this function returns None.
    """

    lines = text.split('\n')

    result = _internal.unwrap(lines, _patterns.MAX_WRAP_LINES,
                    _patterns.MIN_HEADER_LINES,_patterns.MIN_QUOTED_LINES)
    if result:
        typ, top_range, hdrs, main_range, bottom_range, needs_unindent = result

        text_top = lines[slice(*top_range)] if top_range else ''
        text = lines[slice(*main_range)] if main_range else ''
        text_bottom = lines[slice(*bottom_range)] if bottom_range else ''

        if needs_unindent:
            text = _internal.unindent_lines(text)

        result = {
            'type': typ,
        }

        text = '\n'.join(text).strip()
        text_top = '\n'.join(text_top).strip()
        text_bottom = '\n'.join(text_bottom).strip()

        if text:
            result['text'] = text
        if text_top:
            result['text_top'] = text_top
        if text_bottom:
            result['text_bottom'] = text_bottom

        if hdrs:
            result.update(hdrs)

        return result

def unwrap_html(html):
    """
    If the passed HTML is the HTML body of a forwarded message, a dictionary
    with the following keys is returned:

    - type: "reply", "forward" or "quote"
    - html_top: HTML at the top of the passed message (if found)
    - html_bottom: HTML at the bottom of the passed message (if found)
    - from / to / subject / cc / bcc / reply-to: Corresponding header of the
      forwarded message, if it exists. (if found)
    - html: HTML of the forwarded message (if found)

    Otherwise, this function returns None.
    """
    from . import _html

    tree = _html.get_html_tree(html)

    start_refs, end_refs, lines = _html.get_line_info(tree)

    result = _internal.unwrap(lines, 1, _patterns.MIN_HEADER_LINES, 1)

    if result:
        typ, top_range, hdrs, main_range, bottom_range, needs_unindent = result

        result = {
            'type': typ,
        }

        top_range = _html.trim_slice(lines, top_range)
        main_range = _html.trim_slice(lines, main_range)
        bottom_range = _html.trim_slice(lines, bottom_range)

        if top_range:
            top_tree = _html.slice_tree(tree, start_refs, end_refs, top_range,
                                        html_copy=html)
            html_top = _html.render_html_tree(top_tree)
            if html_top:
                result['html_top'] = html_top

        if bottom_range:
            bottom_tree = _html.slice_tree(tree, start_refs, end_refs,
                                           bottom_range, html_copy=html)
            html_bottom = _html.render_html_tree(bottom_tree)
            if html_bottom:
                result['html_bottom'] = html_bottom

        if main_range:
            main_tree = _html.slice_tree(tree, start_refs, end_refs, main_range)
            if needs_unindent:
                _html.unindent_tree(main_tree)
            html = _html.render_html_tree(main_tree)
            if html:
                result['html'] = html

        if hdrs:
            result.update(hdrs)

    return result



# -*- coding: utf-8 -*-

import re

REPLY_PATTERNS = [
    u'^On (.*) wrote:$', # apple mail/gmail reply
    u'^Am (.*) schrieb (.*):$', # German
    u'^Le (.*) a écrit :$', # French
    u'El (.*) escribió:$', # Spanish
    u'([0-9]{4}/[0-9]{1,2}/[0-9]{1,2}) (.* <.*@.*>)$', # gmail (?) reply
]

REPLY_DATE_SPLIT_REGEX = re.compile(r'^(.*(:[0-9]{2}( [apAP]\.?[mM]\.?)?)), (.*)?$')

FORWARD_MESSAGES = [
    # apple mail forward
    'Begin forwarded message', 'Anfang der weitergeleiteten E-Mail',
    u'Début du message réexpédié', 'Inicio del mensaje reenviado',

    # gmail/evolution forward
    'Forwarded [mM]essage', 'Mensaje reenviado',

    # outlook
    'Original [mM]essage', 'Ursprüngliche Nachricht', 'Mensaje [oO]riginal',

    # Thunderbird forward
    u'Message transféré',
]

# We yield this pattern to simulate Outlook forward styles. It is also used for
# some emails forwarded by Yahoo.
FORWARD_LINE = '________________________________'

FORWARD_PATTERNS = [
    '^{}$'.format(FORWARD_LINE),

] + ['^---+ ?%s ?---+$' % p for p in FORWARD_MESSAGES] \
  + ['^%s:$' % p for p in FORWARD_MESSAGES]

FORWARD_STYLES = [
    # Outlook
    'border:none;border-top:solid #B5C4DF 1.0pt;padding:3.0pt 0in 0in 0in',
]

HEADER_RE = re.compile(r'\*?([-\w ]+):\*?(.*)$', re.UNICODE)

HEADER_MAP = {
    'from': 'from',
    'von': 'from',
    'de': 'from',

    'to': 'to',
    'an': 'to',
    'para': 'to',
    u'à': 'to',
    u'pour': 'to',

    'cc': 'cc',
    'kopie': 'cc',

    'bcc': 'bcc',
    'cco': 'bcc',
    'blindkopie': 'bcc',

    'reply-to': 'reply-to',
    'antwort an': 'reply-to',
    u'répondre à': 'reply-to',
    'responder a': 'reply-to',

    'date': 'date',
    'sent': 'date',
    'received': 'date',
    'datum': 'date',
    'gesendet': 'date',
    'enviado el': 'date',
    'enviados': 'date',
    'fecha': 'date',

    'subject': 'subject',
    'betreff': 'subject',
    'asunto': 'subject',
    'objet': 'subject',
    'sujet': 'subject',
}

COMPILED_PATTERN_MAP = {
    'reply': [re.compile(regex) for regex in REPLY_PATTERNS],
    'forward': [re.compile(regex) for regex in FORWARD_PATTERNS],
}

COMPILED_PATTERNS = sum(COMPILED_PATTERN_MAP.values(), [])

MULTIPLE_WHITESPACE_RE = re.compile('\s+')

# Amount to lines to join to check for potential wrapped patterns in plain text
# messages.
MAX_WRAP_LINES = 2

# minimum number of headers that we recognize
MIN_HEADER_LINES = 2

# minimum number of lines to recognize a quoted block
MIN_QUOTED_LINES = 3

# Characters at the end of line where we join lines without adding a space.
# For example, "John <\njohn@example>" becomes "John <john@example>", but
# "John\nDoe" becomes "John Doe".
STRIP_SPACE_CHARS = '<([{"\''



from setuptools import setup

setup(
    name='quotequail',
    version='0.2',
    url='http://github.com/closeio/quotequail',
    license='MIT',
    author='Thomas Steinacher',
    author_email='engineering@close.io',
    maintainer='Thomas Steinacher',
    maintainer_email='engineering@close.io',
    description='A library that identifies quoted text in plain text and HTML email messages.',
    long_description=__doc__,
    packages=[
        'quotequail',
    ],
    test_suite='tests',
    tests_require=['lxml'],
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Topic :: Communications :: Email',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
